// Copyright (c) 2003 Red Hat
// Written by Graydon Hoare <graydon@redhat.com>

// This file is part of Mauve.

// Mauve is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// Mauve is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Mauve; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.  */

package gnu.testlet;
import java.io.*;

/**
 *
 * This class adapts older Mauve tests, based on the Testlet / TestHarness
 * interfaces, to JUnit's TestCase / TestSuite interfaces. The result is
 * that each Testlet runs as a single TestCase, and the complete collection
 * of all Testlets-mapped-to-TestCases is run as a TestSuite, itself a type
 * of TestCase.
 *
 * we use explicit package names in the source here because we are bridging
 * two packages with very similar vocabulary, and it helps clarify
 * intention.
 *
 */

public class TestletSuite 
  extends junit.framework.TestSuite
{

  private gnu.testlet.TestHarness harness;
  
  private static class JUnitHarness 
    extends gnu.testlet.SimpleTestHarness            
  {  
    // override parts of gnu.testlet.SimpleTestHarness
    public void check (boolean result)
    {
      junit.framework.Assert.assertTrue (result);
    }
    public JUnitHarness()
    {
      super(false,false,false);
    }
  }

  // fill junit.framework.TestSuite with wrapped gnu.testlet.Testlets

  private static class TestletCase extends junit.framework.TestCase
  {
    private gnu.testlet.Testlet testlet;
    private gnu.testlet.TestHarness harness;

    public TestletCase (gnu.testlet.Testlet t,
                        gnu.testlet.TestHarness h)
    {
      super(t.getClass ().getName ());
      testlet = t;
      harness = h;
    }
   
    protected void runTest ()
    {
      testlet.test (harness);
    }    
  }

  public TestletSuite ()
  {
    harness = new JUnitHarness ();
    try 
      {
        BufferedReader r = 
          new BufferedReader 
           (new InputStreamReader
            (new FileInputStream
             (gnu.testlet.config.builddir + File.separator + "classes")));

        while (true)
          {
            String name = null;
            name = r.readLine ();
            if (name == null)
              break;
            try 
              {
                gnu.testlet.Testlet tl = 
                  (gnu.testlet.Testlet) Class.forName (name).newInstance ();
                this.addTest (new TestletCase (tl, harness));
              }
            catch (Exception e)
              {
                System.err.println("Exception building testlet " + name + ": ");
              }
          }
      }
    catch (Exception e)
      {
        System.err.println("Exception building testlet suite: " + e);
      }
  }

  // this is for JUnit's graphical runners to hook in to the class
  public static junit.framework.Test suite () 
  { 
    return new TestletSuite ();
  }
  
  // this is for the command-line to hook into the class
  public static void main (String args[]) { 
    junit.textui.TestRunner.run (suite ());
  }
}
